/******************************************************************************
**
**  Project Name:	DropShell
**     File Name:	DropShell.c
**
**   Description:	Main application code for the QuickShell
**
*******************************************************************************
**                       A U T H O R   I D E N T I T Y
*******************************************************************************
**
**	Initials	Name
**	--------	-----------------------------------------------
**	LDR			Leonard Rosenthol
**	MTC			Marshall Clow
**	SCS			Stephan Somogyi
**
*******************************************************************************
**                      R E V I S I O N   H I S T O R Y
*******************************************************************************
**
**	  Date		Author	Description
**	---------	------	---------------------------------------------
**	23 Jun	94	LDR		Implemented support for disk insertion events
**	02 Feb	94	LDR		Updated for Final SDK & CodeWarrior a2
**						Removed the ResumeProc as per new Apple recommendations
**	11 Dec 93	SCS		Universal Headers/UPPs (Phoenix 68k/PPC & PPCC)
**						Skipped System 6 compatible rev of DropShell source
**	09 Dec 91	LDR		Added support for new "Select File" menu item
**						Quit now sends AEVT to self to be politically correct
**						Added support for the new gSplashScreen
**	24 Nov 91	LDR		Added support for the Apple Menu (duh!)
**	29 Oct 91	SCS		Changes for THINK C 5
**	28 Oct 91	LDR		Officially renamed DropShell (from QuickShell)
**						Added a bunch of comments for clarification
**	06 Oct 91	MTC		Converted to MPW C
**	09 Apr 91	LDR		Added to Projector
**
******************************************************************************/

#include "GifScan.h"
#ifdef ___USE_SIOUX___

#ifndef __MWERKS__
#include <Desk.h>
#include <Dialogs.h>
#include <Errors.h>
#include <Files.h>
#include <Fonts.h>
#include <Memory.h>
#include <Menus.h>
#include <StandardFile.h>
#include <TextEdit.h>
#include <Types.h>
#include <Windows.h>
#endif

#include "DSGlobals.h"
#include "DSUserProcs.h"
#include "DSAppleEvents.h"

#include "DropShell.h"

#include <SIOUX.h>

#include "Prefs.h"

Boolean		gDone, gOApped, gHasAppleEvents, gWasEvent, gCmdHeldDown;
EventRecord	gEvent;
MenuHandle	gAppleMenu, gFileMenu, gEditMenu;
WindowPtr	gSplashScreen;

#ifdef MPW
extern void _DataInit();	
#endif

void InitToolbox (void) 
{

#ifdef MPW
	UnloadSeg ((Ptr) _DataInit );
#endif

	InitGraf ( &qd.thePort );
	InitFonts ();
	InitWindows ();
	InitMenus ();
	TEInit ();
	InitDialogs (NULL);		// use of ResumeProcs no longer approved by Apple
	InitCursor ();
	FlushEvents ( everyEvent, 0 );
	
	// how about some memory fun! Two should be enough!
	MoreMasters ();
	MoreMasters ();
	}

/*
	Let's setup those global variables that the DropShell uses.
	
	If you add any globals for your own use,
	init them in the InitUserGlobals routine in DSUserProcs.c
*/

Boolean InitGlobals (void) 
{
	long aLong;

	gDone			= false;
	gOApped			= false;	// probably not since users are supposed to DROP things!
	gHasAppleEvents	= Gestalt ( gestaltAppleEventsAttr, &aLong ) == noErr;
	gSplashScreen	= NULL;

	return(InitUserGlobals());	// call the user proc
}

/*
	Again, nothing fancy.  Just setting up the menus.
	
	If you add any menus to your DropBox - insert them here!
*/

void SetUpMenus (void) 
{
	gAppleMenu = GetMenu ( kAppleNum );
	AppendResMenu ( gAppleMenu, 'DRVR' );
	InsertMenu ( gAppleMenu, 0 );

	gFileMenu = GetMenu ( kFileNum );
	InsertMenu ( gFileMenu, 0 );
	
	gEditMenu = GetMenu ( kEditNum );
	InsertMenu(gEditMenu, 0);
	
	DrawMenuBar ();
}


/*	--------------- Standard Event Handling routines ---------------------- */

void ShowAbout ( void ) 
{
	(void) Alert ( 128, NULL );
	
}

void DoMenu ( long retVal ) 
{
	short			menuID, itemID;
	Str255			itemStr;
	EventRecord		dummyEvent;

	menuID = HiWord ( retVal );
	itemID = LoWord ( retVal );
	
	switch ( menuID ) {
		case kAppleNum:
			if ( itemID == 1 )
				ShowAbout ();	/*	Show the about box */
			else
			{
				GetMenuItemText(GetMenuHandle(kAppleNum), itemID, itemStr);
				OpenDeskAcc(itemStr);
			}
			break;
			
		case kFileNum:
			switch (itemID)
			{
				case 1:
					SelectFile();	// call file selection userProc
					break;
				
				case 2:		/* Save As */
				
					/* Fake a key event for SIOUX */
					dummyEvent.what = keyDown;
					dummyEvent.modifiers = cmdKey;
					dummyEvent.message = 's' & charCodeMask;
					SIOUXHandleOneEvent(&dummyEvent);
					break;
					
				case 3:		/* Preferences */
					SetPreferences();
					break;
				
				default:
					SendQuitToSelf();	// send self a 'quit' event
					break;
			}
			break;
		
		case kEditNum:
			dummyEvent.what = keyDown;
			dummyEvent.modifiers = cmdKey;
			switch (itemID)
			{
				case 3:		/* Cut */
					dummyEvent.message = 'x' & charCodeMask;
					break;
					
				case 4:		/* Copy */
					dummyEvent.message = 'c' & charCodeMask;
					break;
				
				case 5:		/* Paste */
					dummyEvent.message = 'v' & charCodeMask;
					break;
				
				case 6:		/* Select All */
					dummyEvent.message = 'a' & charCodeMask;
					break;
			}

			SIOUXHandleOneEvent(&dummyEvent);
			break;
			
		default:
			break;
			
		}
	HiliteMenu(0);		// turn it off!
}

void DoMouseDown ( EventRecord *curEvent ) 
{
	WindowPtr	whichWindow;
	short		whichPart;

	whichPart = FindWindow ( curEvent->where, &whichWindow );
	switch ( whichPart ) 
	{
		case inMenuBar:
			DoMenu ( MenuSelect ( curEvent->where ));
			break;
		
		case inSysWindow:
			SystemClick ( curEvent, whichWindow );
			break;
		
		case inDrag:
			{
				Rect	boundsRect = (*GetGrayRgn())->rgnBBox;
				DragWindow ( whichWindow, curEvent->where, &boundsRect );
			}
		default:
			break;
		}
	}



void DoKeyDown ( EventRecord *curEvent ) 
{
	char	theKey;
	short	i;
	char 	commandEdits[] =  { 'X', 'V', 'C', 'A', 'x', 'v', 'c', 'a', 'S', 's' };
	Boolean	alreadyHandled = false;

	theKey = (char) curEvent->message & charCodeMask;
	
	if ( curEvent->modifiers & cmdKey )
	{
		/* Must do something to handle the SIOUX window commands, 
		   filter those commands out. */
		for (i = 0; i < sizeof(commandEdits) / sizeof(char); i++) 
		{
			if ( theKey == commandEdits[i] )
				alreadyHandled = true;
		}
		if ( !alreadyHandled ) DoMenu ( MenuKey (theKey));
	}
}




void main ( void ) 
{
	Boolean		SIOUXDidEvent;
	
	InitToolbox ();
	
	if ( InitGlobals () ) 
	{	
		// if we succeeding in initting self
		if ( !gHasAppleEvents )
		{
			ErrorAlert ( kErrStringID, kCantRunErr, 0, true );
		}
		else
		{
			InitAEVTStuff ();
			SetUpMenus ();
			
			while ( !gDone ) 
			{
				gWasEvent = WaitNextEvent ( everyEvent, &gEvent, 0, NULL );
				
				if ( gWasEvent )
				{
					/* Let SIOUX handle the event first */
					SIOUXDidEvent = SIOUXHandleOneEvent(&gEvent);
				}
				 
				if ( !SIOUXDidEvent )
				{
					switch ( gEvent.what ) 
					{
						case kHighLevelEvent:
							DoHighLevelEvent ( &gEvent );
							break;
							
						case mouseDown:
							DoMouseDown ( &gEvent );
							break;
							
						case keyDown:
						case autoKey:
							DoKeyDown ( &gEvent );
							break;

						case diskEvt:
							if (HiWord(gEvent.message)) 
							{
								Point diskInitPt;
								
								diskInitPt.v = diskInitPt.h = 100;
								DILoad();
								DIBadMount(diskInitPt, gEvent.message);
								DIUnload();
							}
							break;
							
						default:
							break;
						}
					}
				}
			}
		}
	DisposeUserGlobals();	// call the userproc to clean itself up
}

#endif